﻿Imports System.Text.RegularExpressions

Public Class MainWindow

    Private receivedSigKey As String = ""

    Private Sub buttonExecute_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonExecute.Click

        'set temporary files + cleanup
        Dim TempInput = System.IO.Path.GetTempPath + "visualgpg.in"
        Dim TempOutput = System.IO.Path.GetTempPath + "visualgpg.out"
        If System.IO.File.Exists(TempOutput) Then
            System.IO.File.Delete(TempOutput)
        End If
        resetVisibilities()

        ' If signing only, breaks long lines so that bad webmails don't do it themselves later on signed text...
        If ckbSign.Checked And Not ckbEncrypt.Checked Then
            tBoxInput.Text = BreakCodeLines(tBoxInput.Text, 64)
        End If

        ' configure input
        System.IO.File.WriteAllText(TempInput, tBoxInput.Text, System.Text.Encoding.Default)

        ' run GnuPG
        ' source: http://bytes.com/topic/visual-basic-net/answers/757983-controlling-console-application-another-application
        ' other (not tested): http://www.vbdotnetforums.com/console-application/32993-run-console-app-windows-application.html
        ' documentation for ProcessStartInfo: http://msdn.microsoft.com/en-us/library/system.diagnostics.processstartinfo%28v=vs.71%29.aspx
        'Dim start_info As New ProcessStartInfo("cmd.exe", "/c dir")
        'Dim start_info As New ProcessStartInfo("gpg2.exe", "--help")

        Dim start_info As New ProcessStartInfo("gpg2.exe")
        Dim gpgArguments As String = ""

        ' configure arguments
        If ckbDecrypt.Checked Then
            gpgArguments = "--decrypt " + " -o " + TempOutput + " " + TempInput
        Else
            gpgArguments = gpgArguments + "-a "

            If ckbEncrypt.Checked Then
                If ckbSign.Checked Then
                    gpgArguments = gpgArguments + "-se"
                Else
                    gpgArguments = gpgArguments + "-e"
                End If
                If cBoxCrypKey.Text.Length >= 8 Then
                    gpgArguments = gpgArguments + " -r " + cBoxCrypKey.Text
                Else
                    Exit Sub
                End If
            ElseIf ckbSign.Checked Then
                gpgArguments = gpgArguments + "--clearsign"
            Else
                Exit Sub
            End If

            If cBoxSignKey.Text.Length >= 8 Then
                gpgArguments = gpgArguments + " -u " + cBoxSignKey.Text
            End If
            gpgArguments = gpgArguments + " -o " + TempOutput + " " + TempInput
        End If

        'commit arguments
        start_info.Arguments = gpgArguments

        start_info.UseShellExecute = False
        start_info.CreateNoWindow = False
        start_info.RedirectStandardOutput = True
        start_info.RedirectStandardError = True

        ' Make the process and set its start information.
        Dim proc As New Process()
        proc.StartInfo = start_info

        ' Start the process.
        proc.Start()

        ' Attach to stdout and stderr.
        Dim std_out As IO.StreamReader = proc.StandardOutput()
        Dim std_err As IO.StreamReader = proc.StandardError()


        ' Display the GnuPG output
        tBoxGPGOut.Text = "GnuPG output:" + Environment.NewLine + std_err.ReadToEnd + std_out.ReadToEnd()
        'tBoxInput.Text = gpgArguments

        ' Clean up.
        std_out.Close()

        proc.Close()

        ' Display the real result and cleanup
        tBoxOutput.Text = ""
        If System.IO.File.Exists(TempOutput) Then
            tBoxOutput.Text = System.IO.File.ReadAllText(TempOutput, System.Text.Encoding.Default) ' cf http://msdn.microsoft.com/en-us/library/a77w6kkx.aspx
            System.IO.File.Delete(TempOutput)
        End If
        System.IO.File.Delete(TempInput)

        ' if we decrypted, provide some interpretation to GnuPG's output
        If ckbDecrypt.Checked Then
            'Dim RegexSignatureMade As Regex = New Regex("^gpg: Signature made .* using RSA key ID ([0-9a-f]{8})$", RegexOptions.IgnoreCase Or RegexOptions.Multiline)
            'Dim RegexSignatureGood As Regex = New Regex("^gpg: Good signature from ""(.*)""$", RegexOptions.IgnoreCase Or RegexOptions.Multiline)
            'TextBox1.Text = RegexSignatureMade.Replace(tBoxGPGOut.Text, "$1")
            'tBoxOutput.Text = Regex.Replace(tBoxGPGOut.Text, "^gpg: Signature made .* using RSA key ID ([0-9a-f]{8})$", "$1", RegexOptions.IgnoreCase Or RegexOptions.Multiline)
            'Dim RegexCRCError As Regex = New Regex(".*gpg: CRC error.*", RegexOptions.IgnoreCase Or RegexOptions.Singleline)
            'Dim mm As Match = RegexCRCError.Match(tBoxGPGOut.Text)

            'nice about regex: http://msdn.microsoft.com/en-us/library/ms235254.aspx

            ' Check for CRC or other invalid input errors
            If Regex.IsMatch(tBoxGPGOut.Text, ".*gpg: CRC error.*", RegexOptions.IgnoreCase Or RegexOptions.Singleline) Then
                labelOutputStatus.Text = "CRC Error"
                labelOutputStatus.BackColor = Color.Orange
                labelOutputStatus.Visible = True
            ElseIf Regex.IsMatch(tBoxGPGOut.Text, ".*gpg: no valid OpenPGP data found.*", RegexOptions.IgnoreCase Or RegexOptions.Singleline) Then
                labelOutputStatus.Text = "Input doesn't contain OpenPGP data"
                labelOutputStatus.BackColor = Color.Orange
                labelOutputStatus.Visible = True
            ElseIf Regex.IsMatch(tBoxGPGOut.Text, ".*gpg: decrypt_message failed: Unknown system error.*", RegexOptions.IgnoreCase Or RegexOptions.Singleline) Then
                labelOutputStatus.Text = "Decrypt_message failed"
                labelOutputStatus.BackColor = Color.Orange
                labelOutputStatus.Visible = True
            ElseIf Regex.IsMatch(tBoxGPGOut.Text, ".*gpg: BAD signature.*", RegexOptions.IgnoreCase Or RegexOptions.Singleline) Then
                labelOutputStatus.Text = "BAD signature"
                labelOutputStatus.BackColor = Color.Red
                labelOutputStatus.Visible = True
            Else ' Seems like non-erroneous output. Now see if we managed to verify the signature
                receivedSigKey = Regex.Replace(tBoxGPGOut.Text, ".*gpg: Signature made .* using (?:R|D)SA key ID ([0-9a-f]{8}).*", "$1", RegexOptions.IgnoreCase Or RegexOptions.Singleline)
                Dim RegexSignatureGood As Regex = New Regex(".*gpg: Good signature from ""(.*?)"".*", RegexOptions.IgnoreCase Or RegexOptions.Singleline)
                If RegexSignatureGood.IsMatch(tBoxGPGOut.Text) Then
                    labelOutputStatus.Text = "Good signature" + Environment.NewLine + "Key ID " + receivedSigKey
                    ToolTip1.SetToolTip(labelOutputStatus, RegexSignatureGood.Replace(tBoxGPGOut.Text, "$1"))
                    labelOutputStatus.BackColor = Color.Green
                    labelOutputStatus.Visible = True
                ElseIf Regex.IsMatch(tBoxGPGOut.Text, ".*gpg: Can't check signature: No public key.*", RegexOptions.IgnoreCase Or RegexOptions.Singleline) Then
                    labelOutputStatus.Text = "Can't check signature: we don't have the public key"
                    labelOutputStatus.BackColor = Color.Orange
                    labelOutputStatus.Visible = True
                    buttonGetPubKey.Visible = True
                ElseIf Regex.IsMatch(tBoxGPGOut.Text, ".*gpg: encrypted with.*", RegexOptions.IgnoreCase Or RegexOptions.Singleline) Then
                    labelOutputStatus.Text = "Decrypted unsigned message"
                    labelOutputStatus.BackColor = Color.LightGreen
                    labelOutputStatus.Visible = True
                Else
                    labelOutputStatus.Text = "Unknown error"
                    labelOutputStatus.BackColor = Color.Orange
                    labelOutputStatus.Visible = True
                End If
            End If
        End If

        ' if encrypting or signing, add a little line about us ;)
        If ckbSign.Checked And Not ckbEncrypt.Checked Then
            tBoxOutput.Text = Regex.Replace(tBoxOutput.Text, "(-----BEGIN PGP SIGNATURE-----" + Environment.NewLine + "Version:.*" + Environment.NewLine + ")", "$1Comment: Using VisualGPG v" + My.Application.Info.Version.ToString + Environment.NewLine, RegexOptions.IgnoreCase)
        ElseIf ckbEncrypt.Checked Then
            tBoxOutput.Text = Regex.Replace(tBoxOutput.Text, "(-----BEGIN PGP MESSAGE-----" + Environment.NewLine + "Version:.*" + Environment.NewLine + ")", "$1Comment: Using VisualGPG v" + My.Application.Info.Version.ToString + Environment.NewLine, RegexOptions.IgnoreCase)
        End If
    End Sub

    ' Decide whether or not the "execute" button (and other fields) should be enabled
    Private Sub enableDisableControls()
        cBoxCrypKey.Enabled = False
        cBoxSignKey.Enabled = False
        buttonExecute.Enabled = False
        labSignKey.Text = "Sign key"
        ToolTip1.SetToolTip(labSignKey, "Private key to be used for signing." + Environment.NewLine + "If left blank, GnuPG will try to sign with any available private key.")
        If ckbEncrypt.Checked Then
            buttonExecute.Enabled = True
            cBoxCrypKey.Enabled = True
        End If
        If ckbSign.Checked Or ckbDecrypt.Checked Then
            buttonExecute.Enabled = True
            cBoxSignKey.Enabled = True
            If ckbDecrypt.Checked Then
                cBoxSignKey.Enabled = True
                labSignKey.Text = "Decipher key"
                ToolTip1.SetToolTip(labSignKey, "Private key to be used for deciphering." + Environment.NewLine + "If left blank, GnuPG will automatically pick the right key.")
            End If
        End If
    End Sub

    Private Sub resetVisibilities()
        labelOutputStatus.Text = ""
        ToolTip1.SetToolTip(labelOutputStatus, "")
        labelOutputStatus.BackColor = Color.Transparent
        labelOutputStatus.Visible = False
        buttonGetPubKey.Visible = False
    End Sub

    Private Sub ckbSign_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ckbSign.Click
        ckbDecrypt.Checked = False
        enableDisableControls()
    End Sub

    Private Sub ckbEncrypt_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ckbEncrypt.Click
        ckbDecrypt.Checked = False
        enableDisableControls()
    End Sub

    Private Sub MainWindow_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        enableDisableControls()
        resetVisibilities()
    End Sub

    Private Sub ckbDecrypt_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ckbDecrypt.Click
        ckbSign.Checked = False
        ckbEncrypt.Checked = False
        enableDisableControls()
    End Sub

    Private Sub buttonTest_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonTest.Click
        'tBoxOutput.Text = System.IO.File.ReadAllText(System.IO.Path.GetTempPath + "visualgpg.out", System.Text.Encoding.Default)
        Dim start_info As New ProcessStartInfo("gpg2.exe", "--help")

        start_info.UseShellExecute = False
        start_info.CreateNoWindow = False
        start_info.RedirectStandardOutput = True
        start_info.RedirectStandardError = True

        ' Make the process and set its start information.
        Dim proc As New Process()
        proc.StartInfo = start_info

        ' Start the process.
        proc.Start()

        ' Attach to stdout and stderr.
        Dim std_out As IO.StreamReader = proc.StandardOutput()
        Dim std_err As IO.StreamReader = proc.StandardError()


        ' Display the results.
        tBoxGPGOut.Text = "GnuPG output:" + Environment.NewLine + std_err.ReadToEnd + std_out.ReadToEnd()

        ' Clean up.
        std_out.Close()

        proc.Close()
    End Sub

    Private Sub AboutToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutToolStripMenuItem.Click
        AboutBox.Show()
    End Sub

    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click
        End
    End Sub

    Private Sub SettingsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SettingsToolStripMenuItem.Click
        SettingsBox.Show()
    End Sub

    Private Sub buttonCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonCopy.Click
        Clipboard.Clear()
        Clipboard.SetText(tBoxOutput.Text)
    End Sub

    Private Sub ckbUnlockOutput_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ckbUnlockOutput.CheckedChanged
        tBoxOutput.ReadOnly = Not ckbUnlockOutput.Checked
    End Sub

    Private Sub tBoxInput_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tBoxInput.TextChanged
        resetVisibilities() ' If we change text, we should invalidate previous messages such as valid signature, etc. This notably allows for the signature fetcher to perform a recheck.
    End Sub

    Private Sub buttonGetPubKey_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonGetPubKey.Click
        resetVisibilities()
        labelOutputStatus.Text = "Fetching key " + receivedSigKey + "..."
        labelOutputStatus.BackColor = Color.Blue
        labelOutputStatus.Visible = True

        Dim start_info As New ProcessStartInfo("gpg2.exe", "--recv-keys " + receivedSigKey)
        start_info.UseShellExecute = False
        start_info.CreateNoWindow = False
        start_info.RedirectStandardOutput = True
        start_info.RedirectStandardError = True
        Dim proc As New Process()
        proc.StartInfo = start_info
        proc.Start()
        Dim std_out As IO.StreamReader = proc.StandardOutput()
        Dim std_err As IO.StreamReader = proc.StandardError()
        tBoxGPGOut.Text = "GnuPG output:" + Environment.NewLine + std_err.ReadToEnd + std_out.ReadToEnd()
        std_out.Close()
        proc.Close()
        If (Not Regex.IsMatch(tBoxGPGOut.Text, ".*gpg: no valid OpenPGP data found.*", RegexOptions.IgnoreCase Or RegexOptions.Singleline)) And Regex.IsMatch(tBoxGPGOut.Text, ".*gpg: Total number processed: [1-9]{1}[0-9]*.*", RegexOptions.IgnoreCase Or RegexOptions.Singleline) Then
            buttonExecute_Click(buttonGetPubKey, System.EventArgs.Empty)
        Else
            labelOutputStatus.Text = "Key " + receivedSigKey + " not found on the key server. You should probably import it manually."
            labelOutputStatus.BackColor = Color.Orange
            labelOutputStatus.Visible = True
        End If
    End Sub

    Private Sub buttonClearInput_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonClearInput.Click
        tBoxInput.Text = ""
    End Sub

    ' Functions to deal with auto-line breaking
    ' Remotely inspired from http://www.vb-helper.com/howto_break_code.html
    ' Break one line
    Private Function BreakCodeLine(ByVal lineToCut As String, _
        ByVal max_len As Integer) As String
        Dim good_space As Long = 0
        Dim result As String = ""

        ' If the line is short enough, return it already
        If Len(lineToCut) <= max_len Then
            Return lineToCut
        End If

        ' Add line breaks until no more need to be added.
        Do While Len(lineToCut) > max_len
            ' Find the space closest to the maximum character.
            good_space = InStrRev(lineToCut, " ", max_len)
            ' If there are no spaces before MAX_LEN, then cut at MAX_LEN
            If good_space = 0 Then
                good_space = max_len + 1
            End If

            result = result & Mid(lineToCut, 1, good_space - 1) & Environment.NewLine ' moves processed part into result
            lineToCut = LTrim(Mid(lineToCut, good_space)) ' remove the processed part from lineToCut
        Loop
        If Len(lineToCut) > 0 Then
            result = result & lineToCut ' append the remaining characters
        Else ' ...or if none, remove the last line break
            result = Mid(result, 1, Len(result) - Len(Environment.NewLine))
        End If
        Return result
    End Function

    ' Break all of the code's lines.
    Public Function BreakCodeLines(ByVal code As String, ByVal _
        max_len As Integer) As String
        Dim lines As Object
        Dim result As String = ""

        ' Break the code into lines.
        lines = Split(code, Environment.NewLine)

        ' Break the lines.
        For i = LBound(lines) To UBound(lines)
            result = result & BreakCodeLine(lines(i), max_len)
            If i < UBound(lines) Then
                result = result & Environment.NewLine
            End If
        Next i

        Return result
    End Function

End Class
